//
//  DropDownMenu.m
//  weibo
//
//  Created by 曾诗亮 on 2017/1/14.
//  Copyright © 2017年 zsl. All rights reserved.
//

#import "DropDownMenu.h"
#import "Masonry.h"

@interface DropDownMenu () <UITableViewDataSource, UITableViewDelegate>

@property (strong, nonatomic) UILabel *titleLB;
@property (strong, nonatomic) UIImageView *arrowIV;

@property (strong, nonatomic) UITableView *tableView;

@property (strong, nonatomic) UIControl *control;

@end

#define kWidth  [UIScreen mainScreen].bounds.size.width
#define kHeight [UIScreen mainScreen].bounds.size.height

#define kDuration 0.3


@implementation DropDownMenu
{
    NSArray *_titles;
    CGFloat _width;
    
    UIWindow *_window;
    
    BOOL _isMenu;
    
    NSInteger _seletedIndex;
}

- (instancetype)initWithWidth:(CGFloat)width titles:(NSArray *)titles
{
    if (self = [super init]) {
        
        _width = width;
        _titles = titles;
        
        _window = [UIApplication sharedApplication].keyWindow;
        self.frame = CGRectMake(0, 0, 100, 30);
//        self.backgroundColor = [UIColor redColor];
        
        UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(clickSelf:)];
        [self addGestureRecognizer:tap];
        
        [self arrowIV];
        
        _isMenu = NO;
        
        _seletedIndex = -1;
    }
    return self;
}

- (void)clickSelf:(UITapGestureRecognizer *)tap
{
//    NSLog(@"clickSelf");
    
    if (!_isMenu) {
        [UIView animateWithDuration:kDuration animations:^{
            self.arrowIV.transform = CGAffineTransformRotate(self.arrowIV.transform, M_PI);
            self.control.alpha = 0.3f;
            self.tableView.frame = CGRectMake((kWidth-_width)/2, 64, _width, _titles.count*44);
        }];
        
        [self.tableView reloadData];
        
        _isMenu = YES;
        
    }else {
        [self dismissControl];
    }
}

#pragma mark - <UITableViewDataSource>
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return _titles.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"cell"];
    if (!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"cell"];
    }
    
    cell.textLabel.text = _titles[indexPath.row];
    cell.textLabel.textColor = [UIColor whiteColor];
    cell.backgroundColor = [UIColor blackColor];
    
    if (indexPath.row == _seletedIndex) {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    }else{
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    
    return cell;
}

#pragma mark - <UITableViewDelegate>
- (CGFloat)tableView:(UITableView *)tableView estimatedHeightForRowAtIndexPath:(nonnull NSIndexPath *)indexPath
{
    return UITableViewAutomaticDimension;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    [self dismissControl];
    
    self.selectedAtIndex(indexPath.row);
    
    _seletedIndex = indexPath.row;
}

#pragma mark - 懒加载
- (UILabel *)titleLB
{
    if (!_titleLB) {
        _titleLB = [UILabel new];
        _titleLB.text = _titles.firstObject;
        [self addSubview:_titleLB];
        [_titleLB mas_makeConstraints:^(MASConstraintMaker *make) {
            make.center.mas_equalTo(0);
        }];
    }
    return _titleLB;
}

- (UIImageView *)arrowIV
{
    if (!_arrowIV) {
        _arrowIV = [UIImageView new];
        _arrowIV.image = [UIImage imageNamed:@"arrow"];
        [self addSubview:_arrowIV];
        [_arrowIV mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerY.mas_equalTo(0);
            make.left.mas_equalTo(self.titleLB.mas_right).mas_equalTo(5);
        }];
    }
    return _arrowIV;
}

- (UITableView *)tableView
{
    if (!_tableView) {
        _tableView = [[UITableView alloc] initWithFrame:CGRectMake((kWidth-_width)/2, 64, _width, 0) style:UITableViewStylePlain];
        _tableView.dataSource = self;
        _tableView.delegate = self;
        _tableView.tableFooterView = [UIView new];
        _tableView.scrollEnabled = NO;
        [_window addSubview:_tableView];
    }
    return _tableView;
}

- (UIControl *)control
{
    if (!_control) {
        _control = [UIControl new];
        _control.backgroundColor = [UIColor blackColor];
        _control.alpha = 0.0f;
        [_window addSubview:_control];
        [_control mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(64);
            make.left.bottom.right.mas_equalTo(0);
        }];
        [_control addTarget:self action:@selector(clickControl:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _control;
}

- (void)clickControl:(UIControl *)sender
{
    [self dismissControl];
}

- (void)dismissControl
{
    [UIView animateWithDuration:kDuration animations:^{
        self.arrowIV.transform = CGAffineTransformRotate(self.arrowIV.transform, M_PI);
        self.control.alpha = 0.0f;
        self.tableView.frame = CGRectMake((kWidth-_width)/2, 64, _width, 0);
    }];
    
    _isMenu = NO;
}

- (void)selectedAtIndex:(SelectedAtIndex)block
{
    self.selectedAtIndex = block;
}

@end
